import { HatchPattern } from "../../../core/property_mixins";
export declare const cap_lookup: {
    butt: number;
    round: number;
    square: number;
};
export declare const join_lookup: {
    miter: number;
    round: number;
    bevel: number;
};
export declare function hatch_pattern_to_index(pattern: HatchPattern): number;
//# sourceMappingURL=webgl_utils.d.ts.map